/*
 * GalaxyGenerator.h
 *
 *  Created on: Jun 5, 2015
 *      Author: Joakim
 */

#ifndef GALAXYGENERATOR_H_
#define GALAXYGENERATOR_H_

class GalaxyGenerator {
private:
	float3 *positions;
	float3 *velocities;
	float3 *accelerations;
	float *mass;
	int N;
	int solarSystems;
	int solarBodies;

	//For analytic purposes
	float *aphelions;

	void allocateVariables(int N);
public:
	GalaxyGenerator(int solarSystems, int solarBodies);
	float3 findSolarStartPosition(float r, float inclination, float azimuth);
	void findPlanetStartPosition(int primaryIndex, int secondaryIndex, float e, float a, float theta, float azimuth);
	void findStartVelocity(int primaryIndex, int secondaryIndex, float e, float a, float theta, float azimuth);
	void findStartVelocity2D(int primaryIndex, int secondaryIndex, float e, float a, float theta);
	void findStartPosition2D(int primaryIndex, int secondaryIndex, float e, float a, float theta);
	void createGalaxy(int solarSystems, int solarBodies);
	void setSolarSystemValues(int starIndex, int endIndex, float azimuth);

	//CPU
	float3 *getPositions();
	float3 *getVelocities();
	float3 *getAccelerations();
	float *getMasses();

	//Transfers Memory to GPU and deletes the CPU memory
	float3 *getPositionsGPU();
	float3 *getVelocitiesGPU();
	float3 *getAccelerationsGPU();
	float *getMassesGPU();

	int getN();
	int getSolarSystems();
	int getSolarBodies();

};

#endif /* GALAXYGENERATOR_H_ */
