/*
 * SystemGenerator.h
 *
 *  Created on: May 9, 2015
 *      Author: Joakim
 */

#ifndef SYSTEMGENERATOR_H_
#define SYSTEMGENERATOR_H_

#include "../math/orbitalMechanics/TheTwoBodyProblem.h"
#include <vector>

/*
 * Simplified Version of GalaxyGenerator for use with the TwoBodySystem class
 */

class SystemGenerator {
private:

	//Single Precision
	float3 *positions;
	float3 *accelerations;
	float3 *velocities;
	float *masses;

	int N;

	//Allocation
	void allocateVariables(int N);

	//Generator functions
	void setPlanetSolarOrbit(int planetIndex, float planetMass, float a, float e, float startTheta);

public:
	static SystemGenerator* generateSolarSystem(TheTwoBodyProblem *twoBody);
	static SystemGenerator* generateSolarSystem(std::vector<TheTwoBodyProblem*> bodies);
	SystemGenerator(TheTwoBodyProblem *twoBody);
	SystemGenerator(std::vector<TheTwoBodyProblem*> bodies);
	float3* getPositions();
	float3* getAccelerations();
	float3* getVelocities();
	float* getMasses();
	int getN();

};

#endif /* SYSTEMGENERATOR_H_ */
