/*
 * CelestialContants.cpp
 *
 *  Created on: Mar 12, 2015
 *      Author: Joakim
 */

#include "CelestialConstants.h"

/*
 * Constant Type: Mass
 *
 * Definition: Based on Earth Mass
 * 1 Solar Mass equals 332946 Earth Masses
 * The mass of a black hole ranges from 100 to 1000000 Solar Masses
 * The ratio of mass between the other planets in the solar system and the earth are also included
 *
 * Base unit: Earth Mass
 *
 */

//M_e / AU / days
const float CelestialConstants::sun_mass = 332946;
const float CelestialConstants::mercury_mass = 0.0553;
const float CelestialConstants::venus_mass = 0.815;
const float CelestialConstants::earth_mass = 1.0;
const float CelestialConstants::mars_mass = 0.107;
const float CelestialConstants::jupiter_mass = 317.8;
const float CelestialConstants::saturn_mass = 95.2;
const float CelestialConstants::uranus_mass = 14.5;
const float CelestialConstants::neptune_mass = 17.1;
const float CelestialConstants::black_hole_mass = 10 * CelestialConstants::sun_mass;

//Estimated Actual values (too high to use with a renderer)
const float CelestialConstants::black_hole_mass_actual = 4300000 * CelestialConstants::sun_mass;


/*
 * Constant Type: Orbit Eccentricity Information
 *
 * Definition:
 *
 *
 * Base unit: N/A
 *
 */

const float CelestialConstants::mercury_e = 0.205;
const float CelestialConstants::venus_e = 0.007;
const float CelestialConstants::earth_e = 0.017;
const float CelestialConstants::mars_e = 0.094;
const float CelestialConstants::jupiter_e = 0.049;
const float CelestialConstants::saturn_e = 0.057;
const float CelestialConstants::uranus_e = 0.046;
const float CelestialConstants::neptune_e = 0.011;

/*
 * Constant Type: Orbit semi-major axis a Information
 *
 * Definition:
 *
 * Base unit: 1 AU (Astronomical Unit, average distance from the earth to the sun)
 *
 */

//M_e / AU / days
const float CelestialConstants::mercury_a = 0.387;
const float CelestialConstants::venus_a = 0.723;
const float CelestialConstants::earth_a = 1.0;
const float CelestialConstants::mars_a = 1.524;
const float CelestialConstants::jupiter_a = 5.204;
const float CelestialConstants::saturn_a = 9.582;
const float CelestialConstants::uranus_a = 19.201;
const float CelestialConstants::neptune_a = 30.047;

/*
 *
 * Constant Type: Distance (used in galaxy generator)
 *
 *
 * Base unit: 1 AU
 *
 */

const float CelestialConstants::black_hole_distance_actual = 1.64423288E+9;
const float CelestialConstants::black_hole_distance = 500;

/*
 * UNUSED CONSTANTS
 *
 */

/* G: 6.67384E-11 kg/m/s
const float CelestialConstants::earth_a = 152.1E9;
const float CelestialConstants::mercury_a = earth_a * 0.39;
const float CelestialConstants::venus_a = earth_a * 0.723;
const float CelestialConstants::mars_a = earth_a * 1.524;
const float CelestialConstants::jupiter_a = earth_a * 5.203;
const float CelestialConstants::saturn_a = earth_a * 9.539;
const float CelestialConstants::uranus_a = earth_a * 19.18;
const float CelestialConstants::neptune_a = earth_a * 30.06;
*/

/* G: 6.67384E-11 kg/m/s
const float CelestialConstants::earth_mass = 5.972E24;
const float CelestialConstants::sun_mass = earth_mass * 332946.0;
const float CelestialConstants::mercury_mass = earth_mass * 0.0553;
const float CelestialConstants::venus_mass = earth_mass * 0.815;
const float CelestialConstants::mars_mass = earth_mass * 0.107;
const float CelestialConstants::jupiter_mass = earth_mass * 317.8;
const float CelestialConstants::saturn_mass = earth_mass * 95.2;
const float CelestialConstants::uranus_mass = earth_mass * 14.5;
const float CelestialConstants::neptune_mass = earth_mass * 17.1;
const float CelestialConstants::black_hole_mass = sun_mass * 10.0;
*/


