/*
 * UserInput.cpp
 *
 *  Created on: Feb 18, 2015
 *      Author: Joakim
 */
#include "UserInput.h"
#include <stdio.h>
#include "CelestialConstants.h"

//Nbody Algorithm
NbodyAlgorithm UserInput::nbodyAlgorithm = NbodyAlgorithm::AllPairsCPU;
NbodyAlgorithm UserInput::fallbackAlgorithm;

float UserInput::pauseSimulationAtTimestep = 0;

//Integration Options
IntegrationMethod UserInput::integrationMethod = IntegrationMethod::SymplecticEuler;
float  UserInput::celestialIntegrationRatio = 2.0;

float UserInput::blackHoleClosestDistance = 100;
float UserInput::blackHoleMass = 10 * CelestialConstants::sun_mass;
int UserInput::galaxyPlanes = 1;
float UserInput::galaxyPlanesDistance = 30;

//Threshold
bool UserInput::enableThresholdSwitching = false;
float UserInput::distanceThreshold = 0;

bool UserInput::measurePerformance = false;

//GPU Block Sizes
int UserInput::allPairsBlockSize = 32;
int UserInput::celestialNbodyBlockSize = 32;
int UserInput::integrationBlockSize = 32;

//Miscellaneous Options
bool UserInput::logSimulation = false;
int UserInput::res_width = 600;
int UserInput::res_height = 600;


//Setter

void UserInput::setBlackHoleMass(float starRatio) {

	UserInput::blackHoleMass = starRatio * CelestialConstants::sun_mass;

}

bool UserInput::setNbodyAlgorithm(NbodyAlgorithm nbodyAlgorithm) {

	bool useGPU;

	if (nbodyAlgorithm == NbodyAlgorithm::AllPairsCPU || nbodyAlgorithm == NbodyAlgorithm::CelestialNbodyCPU) {
		useGPU = false;
	} else {
		useGPU = true;
	}

	UserInput::nbodyAlgorithm = nbodyAlgorithm;

	return useGPU;

}

bool UserInput::setIntegrationMethod(IntegrationMethod integrationMethod) {

	bool setFallBack = false;

	if (integrationMethod == IntegrationMethod::CelestialIntegrationExplicit ||
			integrationMethod == IntegrationMethod::CelestialIntegrationSymplectic) {
		setFallBack = true;
	}

	UserInput::integrationMethod = integrationMethod;

	return setFallBack;

}

bool UserInput::setupSimulation(NbodyAlgorithm nbodyAlgorithm, IntegrationMethod integrationMethod) {

	bool setFallback = setIntegrationMethod(integrationMethod);

	bool useGPU = false;
	useGPU = UserInput::setNbodyAlgorithm(nbodyAlgorithm);

	if (setFallback) {
		UserInput::fallbackAlgorithm = nbodyAlgorithm;
	} else if (useGPU) {
		UserInput::fallbackAlgorithm = NbodyAlgorithm::AllPairsGPU;
	} else {
		UserInput::fallbackAlgorithm = NbodyAlgorithm::AllPairsCPU;
	}

	return useGPU;

}

void UserInput::enableThreshold(float threshold) {

	if (threshold > 0) {
		UserInput::enableThresholdSwitching = true;
		UserInput::distanceThreshold = threshold;
	}

}

void UserInput::enableCelestialIntegration(float integrationRatio) {

	UserInput::nbodyAlgorithm = NbodyAlgorithm::CelestialNbodyGPU;
	UserInput::celestialIntegrationRatio = integrationRatio;

}

void UserInput::resetToDefault() {

	UserInput::nbodyAlgorithm = NbodyAlgorithm::AllPairsCPU;
	UserInput::integrationMethod = IntegrationMethod::SymplecticEuler;
	UserInput::celestialIntegrationRatio = 2.0;
	UserInput::enableThresholdSwitching = false;
	UserInput::distanceThreshold = 0;
	UserInput::measurePerformance = false;
	UserInput::allPairsBlockSize = 32;
	UserInput::celestialNbodyBlockSize = 32;
	UserInput::integrationBlockSize = 32;
	UserInput::logSimulation = false;
	UserInput::res_width = 600;
	UserInput::res_height = 600;

}

