#include "IntegrationModule.h"
#include "SimpleIntegration.h"
#include "SymplecticEuler.h"
#include "IntegrationMethod.h"
#include <stdio.h>
#include "../../system/CelestialSystem.h"
#include "../../input/UserInput.h"

void IntegrationModule::performIntegration(CelestialSystem *system, float dt) {

	if (UserInput::logSimulation) {
		printf("[Log] Starting Integration with timestep %.2f...\n", dt);
	}

	if (system->isUsingGPU()) {
		performIntegrationGPU(system, dt);
	} else {
		performIntegrationCPU(system, dt);
	}

}

void IntegrationModule::performIntegrationCPU(CelestialSystem *system, float dt) {

	if (UserInput::logSimulation) {
		printf("[Log] The Integration will be performed on the CPU...\n");
	}

	bool thresholdReached;
	if (system->getCurrentlyEnabledIntegrationMethod() == IntegrationMethod::CelestialIntegrationExplicit) {
		thresholdReached = SimpleIntegration::performCelestialIntegrationCPU(system, dt);
	} else if (system->getCurrentlyEnabledIntegrationMethod() == IntegrationMethod::CelestialIntegrationSymplectic) {
		thresholdReached = SymplecticEuler::performCelestialIntegrationCPU(system, dt);
	} else if (system->getCurrentlyEnabledIntegrationMethod() == IntegrationMethod::ExplicitEuler) {
		thresholdReached = SimpleIntegration::performIntegrationCPU(system, dt);
	} else { //Default to Symplectic Euler Integration
		thresholdReached = SymplecticEuler::performIntegrationCPU(system, dt);
	}

	system->setThresholdReached(thresholdReached);

}

void IntegrationModule::performIntegrationGPU(CelestialSystem *system, float dt) {

	if (UserInput::logSimulation) {
		printf("[Log] The Integration will be performed on the GPU...\n");
	}

	bool thresholdReached;
	if (system->getCurrentlyEnabledIntegrationMethod() == IntegrationMethod::CelestialIntegrationExplicit) {
		thresholdReached = SimpleIntegration::performCelestialIntegrationGPU(system, dt);
	} else if (system->getCurrentlyEnabledIntegrationMethod() == IntegrationMethod::CelestialIntegrationSymplectic) {
		thresholdReached = SymplecticEuler::performCelestialIntegrationGPU(system, dt);
	} else if (system->getCurrentlyEnabledIntegrationMethod() == IntegrationMethod::ExplicitEuler) {
		thresholdReached = SimpleIntegration::performIntegrationGPU(system, dt);
	} else { //Default to Symplectic Euler Integration
		thresholdReached = SymplecticEuler::performIntegrationGPU(system, dt);
	}

	system->setThresholdReached(thresholdReached);

}
