/*
 * TheTwoBodyProblem.h
 *
 *  Created on: May 18, 2015
 *      Author: Joakim
 */

#ifndef THETWOBODYPROBLEM_H_
#define THETWOBODYPROBLEM_H_

#include <vector>

class TheTwoBodyProblem {

private:

	//Output Variable
	double3 planetPos;

	//Start Variables
	double starMass;
	double planetMass;
	double3 planetStartVelocity;
	double3 planetStartPosition;

	//Orbit information
	double e; //Eccenctricity
	double a; //Ellipse semi-major axis length
	double T; //Orbital Period
	double n; //Mean Motion
	double r_0; //Start Distance from m1
	int numOrbits;

	//Time variables
	double t; //Current time
	double t_peri; //Time since pericentre passage
	double dt;

	std::vector<float> errorOverTime;

	double calculateMeanAnomality(double n, double t, double t_peri);
	double calculateMeanMotion(double T);
	double calculateOrbitalPeriod(double a, double phi);
	double approximateEccentricAnomaly(double e, double M, double E_0);
	void printOrbitalInfo();
	void printSimulationError(float3 simulationPos, int body_i);

public:
	TheTwoBodyProblem(double starMass, double planetMass, float dt);
	void setStartPosition(double theta);
	void setSimulationPosition(int body_i, double x, double y, double z);
	void setOrbit(double e, double a);
	void setTimestep(double dt);
	void moveOrbitingPlanetAhead(double dt, bool useLog, float3 simulationPos, int body_i);
	void moveSimulationAhead(double dt, bool useLog);
	void disableAnalyticalRendering();
	double getA();
	double getEccentricity();
	double getMeanMotion();
	double getTimestep();
	double3 getStarPosition();
	double3 getPlanetStartPosition();
	double3 getPlanetStartVelocity();
	double3 getPlanetPosition();
	static std::vector<TheTwoBodyProblem*> createSolarSystem(float dt);
	std::vector<float> getErrorOverTime();
	double getStarMass();
	double getPlanetMass();
	static void runSolarSystemTestCase(bool shouldShowSimulationReport, float dt);
	static void runTwoBodyTestCase(bool shouldShowSimulationReport, float dt);
	static void runTwoBodyCalculationWithRenderer(double starMass, double planetMass, double e, double a, double startTheta, float dt);
	static void runTwoBodySimulationWithRenderer(double starMass, double planetMass, double e, double a, double startTheta, double dt, bool shouldShowSimulationReport);
	static TheTwoBodyProblem* initializeTwoBodySystem(double starMass, double planetMass, double e, double a, double startTheta, float dt);

};

#endif /* THETWOBODYPROBLEM_H_ */
