/*
 * Celestial3DMath.h
 *
 *  Created on: Apr 6, 2015
 *      Author: Joakim
 */

#ifndef CELESTIAL3DMATH_H_
#define CELESTIAL3DMATH_H_

#include "math.h"
#include <stdio.h>

#define ToRadian(x) ((x) * M_PI / 180.0f)
#define ToDegree(x) ((x) * 180.0f / M_PI)

//Methods

PMatrix4f CreateIdentityMatrix() {

	PMatrix4f identity;
	identity.m[0][0] = 1.0f; identity.m[0][1] = 0.0f; identity.m[0][2] = 0.0f; identity.m[0][3] = 0.0f;
	identity.m[1][0] = 0.0f; identity.m[1][1] = 1.0f; identity.m[1][2] = 0.0f; identity.m[1][3] = 0.0f;
	identity.m[2][0] = 0.0f; identity.m[2][1] = 0.0f; identity.m[2][2] = 1.0f; identity.m[2][3] = 0.0f;
	identity.m[3][0] = 0.0f; identity.m[3][1] = 0.0f; identity.m[3][2] = 0.0f; identity.m[3][3] = 1.0f;

	return identity;

}

PMatrix4f CreateLookAtMatrix(PVector3f target, PVector3f cameraPos, PVector3f cameraUp) {

	PVector3f F = target - cameraPos;
	PVector3f f = F.GetNormalization();
	PVector3f UP = cameraUp.GetNormalization();
	PVector3f s = f.Cross(UP);
	PVector3f s_normalized = s.GetNormalization();
	PVector3f u = s_normalized.Cross(f);

	PMatrix4f lookAt;
	lookAt.m[0][0] = s.x;  lookAt.m[0][1] = s.y;  lookAt.m[0][2] = s.z;  lookAt.m[0][3] = 0.0f;
	lookAt.m[1][0] = u.x;  lookAt.m[1][1] = u.y;  lookAt.m[1][2] = u.z;  lookAt.m[1][3] = 0.0f;
	lookAt.m[2][0] = -f.x; lookAt.m[2][1] = -f.y; lookAt.m[2][2] = -f.z; lookAt.m[2][3] = 0.0f;
	lookAt.m[3][0] = 0.0f; lookAt.m[3][1] = 0.0f; lookAt.m[3][2] = 0.0f; lookAt.m[3][3] = 1.0f;

	return lookAt;

}

PMatrix4f CreateScaleMatrix(PVector3f scaleVector) {

	PMatrix4f scale;
	scale.m[0][0] = scaleVector.x; scale.m[0][1] = 0.0f; scale.m[0][2] = 0.0f; scale.m[0][3] = 0.0f;
	scale.m[1][0] = 0.0f; scale.m[1][1] = scaleVector.y; scale.m[1][2] = 0.0f; scale.m[1][3] = 0.0f;
	scale.m[2][0] = 0.0f; scale.m[2][1] = 0.0f; scale.m[2][2] = scaleVector.z; scale.m[2][3] = 0.0f;
	scale.m[3][0] = 0.0f; scale.m[3][1] = scaleVector.y; scale.m[3][2] = 0.0f; scale.m[3][3] = 1.0f;

	return scale;

}

//NOT IN USE
PMatrix4f CreateRotateMatrix(PVector3f rotateVector) { //Rotates around the Z-axis

	PMatrix4f rotate;

	return rotate;

}

PMatrix4f CreateTranslateMatrix(PVector3f translateVector) {

	PMatrix4f translate;
	translate.m[0][0] = 1.0f; translate.m[0][1] = 0.0f; translate.m[0][2] = 0.0f; translate.m[0][3] = translateVector.x;
	translate.m[1][0] = 0.0f; translate.m[1][1] = 1.0f; translate.m[1][2] = 0.0f; translate.m[1][3] = translateVector.y;
	translate.m[2][0] = 0.0f; translate.m[2][1] = 0.0f; translate.m[2][2] = 1.0f; translate.m[2][3] = translateVector.z;
	translate.m[3][0] = 0.0f; translate.m[3][1] = 0.0f; translate.m[3][2] = 0.0f; translate.m[3][3] = 1.0f;

	return translate;

}

PMatrix4f CreateTranslatedScaledMatrix(PVector3f translateVector, PVector3f scaleVector) {

	PMatrix4f translate;
	translate.m[0][0] = scaleVector.x; translate.m[0][1] = 0.0f; translate.m[0][2] = 0.0f; translate.m[0][3] = translateVector.x;
	translate.m[1][0] = 0.0f; translate.m[1][1] = scaleVector.y; translate.m[1][2] = 0.0f; translate.m[1][3] = translateVector.y;
	translate.m[2][0] = 0.0f; translate.m[2][1] = 0.0f; translate.m[2][2] = scaleVector.z; translate.m[2][3] = translateVector.z;
	translate.m[3][0] = 0.0f; translate.m[3][1] = 0.0f; translate.m[3][2] = 0.0f; translate.m[3][3] = 1.0f;

	return translate;
}

PMatrix4f CreatePerspectiveMatrix(PerspectiveValues p_val) {

	const float ratio = p_val.width / p_val.height;
	const float zRange = p_val.zNear - p_val.zFar;
	const float tanHalfFOV = tanf(ToRadian(p_val.FOV / 2.0));

	PMatrix4f perspective;
	perspective.m[0][0] = 1.0f / (tanHalfFOV * ratio);
	perspective.m[0][1] = 0.0f;
	perspective.m[0][2] = 0.0f;
	perspective.m[0][3] = 0.0f;

	perspective.m[1][0] = 0.0f;
	perspective.m[1][1] = 1.0f / tanHalfFOV;
	perspective.m[1][2] = 0.0f;
	perspective.m[1][3] = 0.0f;

	perspective.m[2][0] = 0.0f;
	perspective.m[2][1] = 0.0f;
	perspective.m[2][2] = (-p_val.zNear - p_val.zFar) / zRange;
	perspective.m[2][3] = 2.0f * p_val.zFar * p_val.zNear / zRange;

	perspective.m[3][0] = 0.0f;
	perspective.m[3][1] = 0.0f;
	perspective.m[3][2] = 1.0f;
	perspective.m[3][3] = 0.0f;

	return perspective;

}

PMatrix4f CreateOrthographicPerspectiveMatrix(OrthographicValues o_val) {

	float left = o_val.left;
	float right = o_val.right;
	float top = o_val.top;
	float bottom = o_val.bottom;
	float near = o_val.near;
	float far = o_val.far;

	float t_x = -1 * (right + left) / (right - left);
	float t_y = -1 * (top + bottom) / (top - bottom);
	float t_z = -1 * (far + near) / (far - near);

	PMatrix4f perspective;
	perspective.m[0][0] = 2 / (right - left);
	perspective.m[0][1] = 0.0f;
	perspective.m[0][2] = 0.0f;
	perspective.m[0][3] = t_x;

	perspective.m[1][0] = 0.0f;
	perspective.m[1][1] = 2 / (top - bottom);
	perspective.m[1][2] = 0.0f;
	perspective.m[1][3] = t_y;

	perspective.m[2][0] = 0.0f;
	perspective.m[2][1] = 0.0f;
	perspective.m[2][2] = -2 / (far - near);
	perspective.m[2][3] = t_z;

	perspective.m[3][0] = 0.0f;
	perspective.m[3][1] = 0.0f;
	perspective.m[3][2] = 0.0f;
	perspective.m[3][3] = 1.0f;

	return perspective;

}

PMatrix4f CreateCameraMatrix(const PVector3f& target, const PVector3f& up) {

	PVector3f N = target;
	N.Normalize();
	PVector3f U = up;
	U.Normalize();
	U = U.Cross(target);
	PVector3f V = N.Cross(U);

	PMatrix4f camera;

	camera.m[0][0] = U.x;
	camera.m[0][1] = U.y;
	camera.m[0][2] = U.z;
	camera.m[0][3] = 0.0f;

	camera.m[1][0] = V.x;
	camera.m[1][1] = V.y;
	camera.m[1][2] = V.z;
	camera.m[1][3] = 0.0f;

	camera.m[2][0] = N.x;
	camera.m[2][1] = N.y;
	camera.m[2][2] = N.z;
	camera.m[2][3] = 0.0f;

	camera.m[3][0] = 0.0f;
	camera.m[3][1] = 0.0f;
	camera.m[3][2] = 0.0f;
	camera.m[3][3] = 0.0f;

	return camera;

}


#endif /* CELESTIAL3DMATH_H_ */
