/*
 * CelestialStructs.h
 *
 *  Created on: Apr 6, 2015
 *      Author: Joakim
 */

#ifndef CELESTIALSTRUCTS_H_
#define CELESTIALSTRUCTS_H_

#include "math.h"
#include <stdio.h>

struct PVector3f {

	float x;
	float y;
	float z;

	PVector3f()
	{
		x = 0;
		y = 0;
		z = 0;
	}

	PVector3f(float _x, float _y, float _z)
	{
		x = _x;
		y = _y;
		z = _z;
	}

	inline PVector3f operator+(const PVector3f& v) const {

		const float _x = x + v.x;
		const float _y = y + v.y;
		const float _z = z + v.z;

		return PVector3f(_x, _y, _z);

	}

	inline PVector3f operator-(const PVector3f& v) const {

		const float _x = x - v.x;
		const float _y = y - v.y;
		const float _z = z - v.z;

		return PVector3f(_x, _y, _z);

	}

	inline void Print() {

		printf("X: %f Y: %f Z: %f \n", x, y, z);

	}


	PVector3f Cross(const PVector3f& v) const
	{

	    const float _x = y * v.z - z * v.y;
	    const float _y = z * v.x - x * v.z;
	    const float _z = x * v.y - y * v.x;

	    return PVector3f(_x, _y, _z);
	}

	float Distance(const PVector3f& v) const
	{

		return 0;

	}

	PVector3f GetNormalization() {

	    const float Length = sqrtf(x * x + y * y + z * z);

	    const float _x = x / Length;
	    const float _y = y / Length;
	    const float _z = z / Length;

	    return PVector3f(_x, _y, _z);
	}

	PVector3f& Normalize()
	{
	    const float Length = sqrtf(x * x + y * y + z * z);

	    if (Length != 0) { //Illegal action?
		    x /= Length;
		    y /= Length;
		    z /= Length;
	    }


	    return *this;
	}

};

struct PMatrix4f {
	float m[4][4];

	inline PMatrix4f operator*(const PMatrix4f& rightMatrix) const //m is the left matrix
	{
	    PMatrix4f resultMatrix;
	    for (int i = 0;i < 4;i++) {
	       for (int j = 0; j < 4;j++) {
	    	   resultMatrix.m[i][j] = m[i][0] * rightMatrix.m[0][j] +
	    			   	   	   	    m[i][1] * rightMatrix.m[1][j] +
	    			   	   	   	    m[i][2] * rightMatrix.m[2][j] +
	    			   	   	   	    m[i][3] * rightMatrix.m[3][j];
	       }
	    }

	    return resultMatrix;
	}

	PMatrix4f() {

		m[0][0] = 1.0f; m[0][1] = 0.0f; m[0][2] = 0.0f; m[0][3] = 0.0f;
		m[1][0] = 0.0f; m[1][1] = 1.0f; m[1][2] = 0.0f; m[1][3] = 0.0f;
		m[2][0] = 0.0f; m[2][1] = 0.0f; m[2][2] = 1.0f; m[2][3] = 0.0f;
		m[3][0] = 0.0f; m[3][1] = 0.0f; m[3][2] = 0.0f; m[3][3] = 1.0f;


	}

};

struct PerspectiveValues { //Used to create a perspective projection

	float width;
	float height;
	float FOV;
	float zNear;
	float zFar;

};

struct OrthographicValues { //Used to create an orthographic projection

	float left;
	float right;
	float top;
	float bottom;
	float width;
	float height;
	float near;
	float far;

};


#endif /* CELESTIALSTRUCTS_H_ */
