/*
 * TransformationPipeline.cpp
 *
 *  Created on: Apr 3, 2015
 *      Author: Joakim
 */

#include "TransformationPipeline.h"
#include "Celestial3DMath.h"

//Assign the temporary vector values

void TransformationPipeline::scale(float x, float y, float z) {

	scaleVector.x = x;
	scaleVector.y = y;
	scaleVector.z = z;

}

void TransformationPipeline::rotate(float x, float y, float z) {

	rotateVector.x = x;
	rotateVector.y = y;
	rotateVector.z = z;

}

void TransformationPipeline::translate(float x, float y, float z) {

	translateVector.x = x;
	translateVector.y = y;
	translateVector.z = z;

}

void TransformationPipeline::setPerspectiveValues(float width, float height,
		float FOV, float zNear, float zFar) {

	PerspectiveValues p_val;
	p_val.width = width;
	p_val.height = height;
	p_val.FOV = FOV;
	p_val.zNear = zNear;
	p_val.zFar = zFar;

	perspectiveValues = p_val;

}

void TransformationPipeline::setOrthographicValues(float width, float height, float left, float right, float top, float bottom, float near, float far) {

	OrthographicValues o_val;
	o_val.width = width;
	o_val.height = height;
	o_val.left = left;
	o_val.right = right;
	o_val.top = top;
	o_val.bottom = bottom;
	o_val.near = near;
	o_val.far = far;

	orthographicValues = o_val;

}

// Get the Final Matrix

const PMatrix4f* TransformationPipeline::getTransformation(PVector3f cameraPos, PVector3f cameraTarget, PVector3f cameraUp) {

	//No need to transform world, as we calculate it using the nbody algorithm
	PMatrix4f world_transform = CreateIdentityMatrix();

	//View Transform (Camera Position and orientation)
	PMatrix4f camera_orientation = CreateLookAtMatrix(cameraTarget, cameraPos, cameraUp);
	PMatrix4f view_transform = camera_orientation;

	//Projection transform
	PMatrix4f projection_transform = CreateOrthographicPerspectiveMatrix(orthographicValues);

	//WorldViewProjection Matrix(WVP) AKA ModelViewProjection(MVP)
	transformationMatrix = projection_transform * view_transform * world_transform;

	return &transformationMatrix;

}

