/*

Contains the methods needed for serial body-body calculations

*/

//============================================
// include guard
#ifndef NBODY_BODY_INTERACTIONS_H_
#define NBODY_BODY_INTERACTIONS_H_

//============================================

//CPU and GPU
__host__ __device__ float3 computeBodyBody(float3 i_pos, float i_mass, float3 j_pos, float j_mass, float3 i_acc);
__host__ __device__ bool distanceIsZero(float3 distanceVector);
__host__ __device__ float3 calculateDistanceVector(float3 pos_i, float3 pos_j);
__host__ __device__ float3 calculateAccelerationChange(float j_mass, float absoluteDistanceCubed,
		float3 distanceVector);
__host__ __device__ float calculateAbsoluteDistance(float3 distanceVector);
__host__ __device__ float3 sumAccelerationChange(float3 i_acc, float3 dA);
__host__ __device__ float4 calculateMassCenter(float3 *position, float *mass, int startIndex, int endIndex);

__device__ float3 computeBodyBodyGPU(float3 i_pos, float i_mass, float3 j_pos, float j_mass, float3 i_acc);

__global__ void calculateMassCenterForSolarSystems(float3 *position, float *mass, float4 *output_com);
float4 *calculateMassCenterForSolarSystemsCPU(float3 *position, float *mass, int solarSystems, int solarBodies);
__global__ void calculateExternalAccelerationGPU(float3 *acceleration_d, float3 *position_d, float *mass_d, float4 *massCenters,
		int solarSystems, int solarBodies, bool sumAcc, float distanceThreshold, bool *thresholdReached);

#endif //NBODY_BODY_INTERACTIONS_H_
