/*
 * CelestialRenderer.h
 *
 *  Created on: Jun 10, 2015
 *      Author: Joakim
 */

#ifndef CELESTIALRENDERER_H_
#define CELESTIALRENDERER_H_

//Includes for SDL
#include <GL/glew.h>
#include <SDL2/SDL.h>
#include <SDL2/SDL_opengl.h>

#include "../math/rendering/CelestialStructs.h"
#include "../system/CelestialSystem.h"
#include "../math/rendering/TransformationPipeline.h"
#include <vector>

class CelestialRenderer {

public:
	CelestialRenderer(CelestialSystem *renderingSystem, float dt);
	void setupAndRun(int screenWidth, int screenHeight);

private:

	//System to render
	CelestialSystem *renderingSystem;
	float dt;

	//Transformation Pipeline
	TransformationPipeline *pipeline;

	//Renderer and window settings variables
	SDL_Window *window;
	SDL_GLContext glContext;
	SDL_Renderer *renderer;
	bool shouldQuit;
	bool runSimulation;
	GLuint shaderProgram;
	GLuint gWorldID;

	//Variables to determine camera position
	float camDistance;
	float camHorizontal;
	float camVertical;

	//Camera Vectors
	PVector3f cameraPos;
	PVector3f cameraTarget;
	PVector3f cameraOrientation;

	double fov;
	int screen_width;
	int screen_height;

	//Main Loop
	void runRenderingLoop();

	//Input Functions
	void handleInput();

	//Render Functions
	void renderBodies();
	void prepareToRender();

	//Camera Functions
	void selectSystem(int system_i);

	void zoomCamera(float offset);
	void moveCameraHorizontally(float angleOffset);
	void moveCameraVertically(float angleOffset);
	void setCameraOrientation(PVector3f cameraOrientation);
	void setCamera(PVector3f pos, PVector3f target, PVector3f orientation);
	void setCameraOrbitPosition();
	void setProjection();
	void setFOV(float fov);

	//Object Move Functions
	void moveBodies();
	void toggleRunSimulation();

	//Analysis
	void graphMovement();
	std::vector<float3*> positionData;
	float dataRecordInterval;
	float timeSinceLastDataRecord;

};

#endif /* CELESTIALRENDERER_H_ */
