/*
 * SimpleRenderer.h
 *
 *  Created on: May 22, 2015
 *      Author: Joakim
 */

#ifndef SIMPLERENDERER_H_
#define SIMPLERENDERER_H_

#include <GL/glew.h>
#include <SDL2/SDL.h>
#include <SDL2/SDL_opengl.h>
#include <vector>
#include "../math/orbitalMechanics/TheTwoBodyProblem.h"
#include "../system/CelestialSystem.h"


class SimpleRenderer {
private:

	//Bodies Information
	std::vector<TheTwoBodyProblem*> bodies;
	CelestialSystem *simulationSystem;

	//Rendering Variables
	bool renderSimulation;
	bool renderAnalytical;
	bool shouldShowSimulationReport;
	int screen_width;
	int screen_height;
	float dt;
	bool shouldQuit;
	SDL_Window *window;

	//Rendering Functions
	void renderBodies();
	void renderEllipse();

	//Input
	void handleInput();

	//Movement
	void moveBodies();

	//Main Loop
	void runRenderingLoop();


public:
	SimpleRenderer(TheTwoBodyProblem *twoBody, float dt, bool shouldShowSimulationReport);
	SimpleRenderer(std::vector<TheTwoBodyProblem*> bodies, float dt, bool shouldShowSimulationReport);
	SimpleRenderer(std::vector<TheTwoBodyProblem*> bodies, bool renderSimulation, bool renderAnalytical, float dt, bool shouldShowSimulationReport);
	SimpleRenderer(CelestialSystem *simulationSystem, float dt);
	~SimpleRenderer();
	void setupAndRun(int screenWidth, int screenHeight);
	void showReport();

};

#endif /* SIMPLERENDERER_H_ */
