#ifndef CUDAERRORCHECK_H_
#define CUDAERRORCHECK_H_

#include <stdlib.h>
#include <stdio.h>

#define CUDA_ERROR_CHECK //Define to enable error checking

#define CudaSafeCall( err ) __cudaSafeCall( err, __FILE__, __LINE__ )
#define CudaCheckError()    __cudaCheckError( __FILE__, __LINE__ )

//Inline Method Declaration
inline void __cudaSafeCall( cudaError err, const char *file, const int line );
inline void __cudaCheckError( const char *file, const int line );

//Inline Method Implementation
inline void __cudaSafeCall( cudaError err, const char *file, const int line ){
#ifdef CUDA_ERROR_CHECK
    if (cudaSuccess != err) {
        fprintf( stderr, "cudaSafeCall() failed at %s:%i : %s\n",
                 file, line, cudaGetErrorString(err));
        exit(EXIT_FAILURE);
    }
#endif

    return;
}

//Inline Method Declaration
inline void __cudaCheckError( const char *file, const int line )
{
#ifdef CUDA_ERROR_CHECK

    cudaError err = cudaGetLastError();
    if (cudaSuccess != err) {
        fprintf( stderr, "cudaCheckError() failed at %s:%i : %s\n",
                 file, line, cudaGetErrorString( err ) );
        exit(EXIT_FAILURE);
    }

    err = cudaDeviceSynchronize();
    if( cudaSuccess != err ) {
        fprintf( stderr, "cudaCheckError() with sync failed at %s:%i : %s\n",
                 file, line, cudaGetErrorString( err ) );
        exit(EXIT_FAILURE);
    }

#endif

    return;
}

#endif /* CUDAERRORCHECK_H_ */
