/*
 * OpenglUtility.cpp
 *
 *  Created on: Jun 15, 2015
 *      Author: Joakim
 */

#include "OpenglUtility.h"
#include "Utilities.h"
#include "stdio.h"
#include <stdlib.h>

void OpenglUtility::checkOpenGLError() {

	GLenum error = glGetError();
	while (error != GL_NO_ERROR)
	{
	    if (error == GL_NO_ERROR) {
	    	printf("GL_NO_ERROR\n");
	    } else if (error == GL_INVALID_ENUM) {
	    	printf("GL_INVALID_ENUM\n");
	    } else if (error == GL_INVALID_VALUE) {
	    	printf("GL_INVALID_VALUE\n");
	    } else if (error == GL_INVALID_OPERATION) {
	    	printf("GL_INVALID_OPERATION\n");
	    } else if (error == GL_INVALID_FRAMEBUFFER_OPERATION) {
	    	printf("GL_INVALID_FRAMEBUFFER_OPERATION\n");
	    } else if (error == GL_OUT_OF_MEMORY) {
	    	printf("GL_OUT_OF_MEMORY\n");
	    }
	    error = glGetError();

	}

}

GLuint OpenglUtility::buildShader(GLenum shaderType, const char* filepath) {

	GLuint shader = glCreateShader(shaderType);
	const GLchar *shaderString = Utilities::readFile(filepath);
	glShaderSource(shader, 1, &shaderString, NULL);
	glCompileShader(shader);

	GLint status;
	glGetShaderiv(shader, GL_COMPILE_STATUS, &status);
	if (status == GL_FALSE) {
		printf("Shader %s failed to compile\n", filepath);
		GLint logLength;
		glGetShaderiv(shader, GL_INFO_LOG_LENGTH, &logLength);
		GLchar *log = new GLchar[logLength + 1];
		glGetShaderInfoLog(shader, logLength, NULL, log);
		fputs(log, stderr);
		exit (EXIT_FAILURE);
	}

	return shader;
}

GLuint OpenglUtility::buildLiteralsShader(GLenum shaderType) {

	GLuint shader = glCreateShader(shaderType);
	const GLchar *shaderString;

	if (shaderType == GL_VERTEX_SHADER) {
		shaderString = getVertexShaderLiterals();
	} else {
		shaderString = getFragmentShaderLiterals();
	}

	glShaderSource(shader, 1, &shaderString, NULL);
	glCompileShader(shader);

	GLint status;
	glGetShaderiv(shader, GL_COMPILE_STATUS, &status);
	if (status == GL_FALSE) {
		printf("Shader failed to compile\n");
		GLint logLength;
		glGetShaderiv(shader, GL_INFO_LOG_LENGTH, &logLength);
		GLchar *log = new GLchar[logLength + 1];
		glGetShaderInfoLog(shader, logLength, NULL, log);
		fputs(log, stderr);
		exit (EXIT_FAILURE);
	}

	return shader;

}

GLuint OpenglUtility::buildProgram() {

	/*
	GLuint vertexShader = buildShader(GL_VERTEX_SHADER,
			"src/renderer/shader/vertex_shader.glsl");
	GLuint fragmentShader = buildShader(GL_FRAGMENT_SHADER,
			"src/renderer/shader/fragment_shader.glsl");
	*/

	GLuint vertexShader = buildLiteralsShader(GL_VERTEX_SHADER);
	GLuint fragmentShader = buildLiteralsShader(GL_FRAGMENT_SHADER);

	GLuint shaderProgram = glCreateProgram();
	glAttachShader(shaderProgram, vertexShader);
	glAttachShader(shaderProgram, fragmentShader);
	glLinkProgram(shaderProgram);

//Check for linker errors
	GLint status;
	glGetProgramiv(shaderProgram, GL_LINK_STATUS, &status);
	if (status == GL_FALSE) {
		printf("Program linking failure\n");
		GLint logLength;
		glGetProgramiv(shaderProgram, GL_INFO_LOG_LENGTH, &logLength);
		GLchar *log = new GLchar[logLength + 1];
		glGetProgramInfoLog(shaderProgram, logLength, NULL, log);
		fputs(log, stderr);
		exit (EXIT_FAILURE);
	}

//Cleanup
	glDetachShader(shaderProgram, vertexShader);
	glDetachShader(shaderProgram, fragmentShader);
	glDeleteShader(vertexShader);
	glDeleteShader(fragmentShader);

	return shaderProgram;
}

//Hardcoded shader files. Hard to maintain, but portable

const char* OpenglUtility::getVertexShaderLiterals() {

	const char *vertexShader =
			"#version 410 									\n"
														  " \n"
			"layout(location = 0) in vec3 position; 		\n"
			"uniform mat4 gWorld; 							\n"
			"void main() 									\n"
			"{ 											   \n"
														  "\n"
			"gl_Position = gWorld * vec4(position, 1.0f);  \n"
			"gl_PointSize = 1; 							   \n"
														  "\n"
			"} 											   \n";

	return vertexShader;

}
const char* OpenglUtility::getFragmentShaderLiterals() {

	const char *fragmentShader =
			"#version 410								  \n"
														  "\n"
			"out vec4 outputColor;						  \n"
			"in vec3 colorPosition;					      \n"
			"void main()								  \n"
			"{											  \n"
			"outputColor = vec4(1.0, 1.0, 1.0, 1.0);	  \n"
			"} 										      \n";

	return fragmentShader;

}



