/*
 * Utilities.cpp

 *
 *  Created on: Mar 2, 2015
 *      Author: Joakim
 */
#include <iostream>
#include <fstream>
#include <sstream>
#include <string>
#include <unistd.h>
#include <sys/param.h>
#include <python2.7/Python.h>
#include "../input/UserInput.h"

#include "Utilities.h"

using namespace std;

const char* Utilities::readFile(const char* filepath) {

	char *cwd=NULL;
	size_t size = 0;

	cwd = getcwd(cwd,size);

	char fullpath[MAXPATHLEN];
	strcat(fullpath, cwd);
	strcat(fullpath, "/");
	strcat(fullpath, filepath);

	ifstream in(fullpath);

	if (!in) {
		printf("Error opening file \"%s\"\n", fullpath);
		exit(1);
	}

	std::string fileContent((std::istreambuf_iterator<char>(in)),
			std::istreambuf_iterator<char>());

	return fileContent.c_str();

}

void Utilities::printPythonArray(const char* varName, float *array, int N) {

	printf("%s = [", varName);

	for (int i=0; i<N;i++) {

		printf("%E", array[i]);

		if (i != N-1) {
			printf(", ");
		}

	}

	printf("]\n");

}

std::string Utilities::createPythonArrayString(const char* varName, float *array, int N, int offset) {

	std::ostringstream stream;

	stream << varName << " = [";

	for (int i=offset; i<N;i++) {

		stream << std::scientific << array[i];

		if (i != N-1) {
			stream << ", ";
		}
	}

	stream << "]";

	const std::string arrayString = stream.str();

	return arrayString;

}

void Utilities::runPython3DScatterPlot(float *x, float *y, float *z, int N) {

	  std::string input = createPythonArrayString("x", x, N, 0) + "\n" + createPythonArrayString("y", y, N, 0) +
			  "\n" + createPythonArrayString("z", z, N, 0);

	  Py_Initialize();
	  PyRun_SimpleString("from mpl_toolkits.mplot3d import Axes3D");
	  PyRun_SimpleString("import matplotlib.pyplot as plt");
	  PyRun_SimpleString("fig = plt.figure()");
	  PyRun_SimpleString("ax = fig.add_subplot(111, projection='3d')");
	  PyRun_SimpleString(input.c_str());
	  PyRun_SimpleString("ax.scatter(x, y, z, c='r', marker='.')");
	  PyRun_SimpleString("ax.set_xlabel('Position on the X axis')");
	  PyRun_SimpleString("ax.set_ylabel('Position on the Y axis')");
	  PyRun_SimpleString("ax.set_zlabel('Position on the Z axis')");
	  PyRun_SimpleString("plt.show()");
	  Py_Finalize();

}

void Utilities::runPythonPlotScript(float *x, float *y, int N, int offset, const char* xlabel, const char* ylabel, const char* title) {


	/*
	 std::string output = std::string(xlabel) + "\n" + std::string(ylabel) + "\n" + std::string(title) + "\n" + createPythonArrayString("x", x, N) + "\n" +
			createPythonArrayString("y", y, N);

	*/

	std::string input = createPythonArrayString("x", x, N, offset) + "\n" + createPythonArrayString("y", y, N, offset);
	std::ofstream out("output.txt");
	out << input;
	out.close();

	Py_Initialize();
	PyRun_SimpleString("import pylab as pl");
	PyRun_SimpleString(input.c_str());
	PyRun_SimpleString("pl.plot(x,y)");
	PyRun_SimpleString(xlabel);
	PyRun_SimpleString("pl.xlabel(xlabel)");
	PyRun_SimpleString(ylabel);
	PyRun_SimpleString("pl.ylabel(ylabel)");
	PyRun_SimpleString(title);
	PyRun_SimpleString("pl.title(title)");
	PyRun_SimpleString("pl.show()");
	Py_Finalize();

}

std::string Utilities::IntegrationEnumToText(IntegrationMethod integrationMethod) {

	std::string text;

	if (integrationMethod == IntegrationMethod::ExplicitEuler) {
		text = "Explicit Euler Integration";
	} else if (integrationMethod == IntegrationMethod::SymplecticEuler) {
		text = "Symplectic Euler Integration";
	} else if (integrationMethod == IntegrationMethod::CelestialIntegrationExplicit) {
		text = "Celestial Integration Explicit (Ratio:" + std::to_string(UserInput::celestialIntegrationRatio) + ")";
	} else if (integrationMethod == IntegrationMethod::CelestialIntegrationSymplectic) {
		text = "Celestial Integration Symplectic (Ratio:" + std::to_string(UserInput::celestialIntegrationRatio) + ")";
	} else {
		text = "Name not found";
	}

	return text;

}

std::string Utilities::NbodyEnumToText(NbodyAlgorithm nbodyAlgorithm) {

	std::string text;

	if (nbodyAlgorithm == NbodyAlgorithm::CelestialNbodyCPU) {
		text = "Celestial N-body CPU";
	} else if (nbodyAlgorithm == NbodyAlgorithm::CelestialNbodyGPU) {
		text = "Celestial N-body GPU";
	} else if (nbodyAlgorithm == NbodyAlgorithm::CelestialNbodyGroupedGPU) {
		text = "Celestial N-body Grouped GPU";
	} else if (nbodyAlgorithm == NbodyAlgorithm::AllPairsCPU) {
		text = "N-Body All-Pairs CPU";
	} else if (nbodyAlgorithm == NbodyAlgorithm::AllPairsGPU){
		text = "N-Body All-Pairs GPU";
	} else {
		text = "Name not found";
	}

	return text;
}

double Utilities::get_wtime() {

    struct timeval t;
    gettimeofday(&t, NULL);
    double time_in_mill = (t.tv_sec) * 1000 + (t.tv_usec) / 1000.0;
    return time_in_mill;

}

void Utilities::printElapsedTime(double before, double after) {

	printf("[Simulation Output] CPU Elapsed Time: %f ms\n", after - before);

}
